<h2>Resend unlock instructions</h2>

<%= simple_form_for(resource, as: resource_name, url: unlock_path(resource_name), html: { method: :post }) do |f| %>
  <%= f.error_notification %>
  <%= f.full_error :unlock_token %>

  <div class="form-inputs">
    <%= f.input :email, required: true, autofocus: true %>
  </div>

  <div class="form-actions">
    <%= f.button :submit, "Resend unlock instructions" %>
  </div>
<% end %>

<%= render "devise/shared/links" %>
